<div id="geometrix-container" style="width:100%;height:__HEIGHT__px;"></div>
<style>__CSS__</style>
<script type="importmap">
{
  "imports": {
    "three": "https://cdn.jsdelivr.net/npm/three@0.163.0/build/three.module.js"
  }
}
</script>
<script type="module">
__SCRIPT__
</script>
